package it.openutils.usermanagement.dataobjects;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.Table;

import org.acegisecurity.GrantedAuthority;
import org.acegisecurity.GrantedAuthorityImpl;
import org.acegisecurity.userdetails.UserDetails;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.hibernate.annotations.Fetch;
import org.hibernate.annotations.FetchMode;


/**
 * @author fberar
 * @version $Revision: 80 $ ($Author: fgiust $)
 */
@Entity
@Table(name = "APP_USER")
public class User implements Serializable, UserDetails
{

    /**
     * stable serialVersionUID.
     */
    private static final long serialVersionUID = 1L;

    @Id
    @Column(name = "ID_USER", length = 50)
    private String id;

    @Column(name = "PASSWORD", nullable = false, length = 50)
    private String password;

    @Column(name = "ENABLED", nullable = false)
    private Boolean enabled;

    @Column(name = "NAME", nullable = false, length = 80)
    private String name;

    @Column(name = "SURNAME", nullable = false, length = 80)
    private String surname;

    @Column(name = "EMAIL", nullable = false, length = 150)
    private String email;

    @ManyToMany(cascade = CascadeType.PERSIST, targetEntity = Group.class, fetch = FetchType.EAGER)
    @Fetch(FetchMode.SUBSELECT)
    @JoinTable(name = "APP_USER_GROUP", joinColumns = @JoinColumn(name = "ID_USER"), inverseJoinColumns = @JoinColumn(name = "ID_GROUP"))
    private Set<Group> groups;

    public Set<Group> getGroups()
    {
        return groups;
    }

    public void setGroups(Set<Group> groups)
    {
        this.groups = groups;
    }

    /**
     * Getter for <code>id</code>.
     * @return Returns the id.
     */
    public String getId()
    {
        return this.id;
    }

    /**
     * Setter for <code>id</code>.
     * @param id The id to set.
     */
    public void setId(String id)
    {
        this.id = id;
    }

    public Boolean getEnabled()
    {
        return this.enabled;
    }

    public void setEnabled(Boolean enabled)
    {
        this.enabled = enabled;
    }

    public String getPassword()
    {
        return password;
    }

    public void setPassword(String password)
    {
        this.password = password;
    }

    public Set<Role> getRoles()
    {
        if (groups == null)
        {
            return null;
        }

        Set<Role> roles = new HashSet<Role>();

        for (Group group : groups)
        {
            roles.addAll(group.getRoles());
        }

        return roles;
    }

    /**
     * @see UserDetails.isAccountNonExpired()
     */
    public boolean isAccountNonExpired()
    {
        return (this.enabled == null) ? false : this.enabled.booleanValue();
    }

    /**
     * @see UserDetails.isAccountNonLocked()
     */
    public boolean isAccountNonLocked()
    {
        return (this.enabled == null) ? false : this.enabled.booleanValue();
    }

    /**
     * @see UserDetails.isCredentialsNonExpired()
     */
    public boolean isCredentialsNonExpired()
    {
        return (this.enabled == null) ? false : this.enabled.booleanValue();
    }

    /**
     * @see UserDetails.getAuthorities()
     */
    public GrantedAuthority[] getAuthorities()
    {
        if (this.getRoles() == null)
        {
            return null;
        }

        GrantedAuthority[] auths = new GrantedAuthorityImpl[this.getRoles().size()];
        Iterator<Role> iter = this.getRoles().iterator();
        int i = 0;
        while (iter.hasNext())
        {
            Role r = iter.next();
            auths[i] = new GrantedAuthorityImpl(r.getId());
            i++;
        }

        return auths;
    }

    /**
     * @see UserDetails.getUsername()
     */
    public String getUsername()
    {
        return this.id;
    }

    /**
     * @see UserDetails.isEnabled()
     */
    public boolean isEnabled()
    {
        return (this.enabled == null) ? false : this.enabled.booleanValue();
    }

    public String getName()
    {
        return name;
    }

    public void setName(String name)
    {
        this.name = name;
    }

    public String getSurname()
    {
        return surname;
    }

    public void setSurname(String surname)
    {
        this.surname = surname;
    }

    /**
     * Getter for <code>email</code>.
     * @return Returns the email.
     */
    public String getEmail()
    {
        return this.email;
    }

    /**
     * Setter for <code>email</code>.
     * @param email The email to set.
     */
    public void setEmail(String email)
    {
        this.email = email;
    }

    /**
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString()
    {
        return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE).append("id", this.id).append(
            "enabled",
            this.enabled).append("name", this.name).append("surname", this.surname).append(
            "authorities",
            this.getAuthorities()).append("username", this.getUsername()).append("email", this.email).append(
            "password",
            this.password).toString();
    }

}
